import javax.swing.JFrame;

import PER.rover.Action;
import PER.rover.ActionConstants;
import PER.rover.Log;
import PER.rover.Rover;

/**
 * 
 * PERController represents the main program used to control
 * the PER. 
 * 
 * @author David Lariviere
 */
public class PERController {
	/**
	 * Used to cancel Action currently running within executeAction
	 */
	private boolean stop = false; 
	
	/**
	 * Used to actually control the PER
	 */
	public Rover rov;
	
	
	/**
	 * Entry point of the program. Responsible for displaying GUIs
	 * @param args
	 */
	public static void main(String[] args) {
		PERController perController = new PERController(); 
		JFrame moverGUI = new MoverGUI(perController); 
		moverGUI.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); //exit program when MoverGUI is closed
		perController.connectToPER("192.168.2.112"); 
		//IRScanner.scan(perController, -20, 20, 2); 
	}
	
	public PERController() { 
		rov = new Rover(); 
	}
	
	/** Executes the given Action. If the Action is one that takes pictures,
     * the pictures will be displayed as the Action executes. This method
     * is blocking and does not return until the action is complete.
     * Returns true if and only if the action completes successfully.
     *
     *@returns true if the action completes successfully, false if there is
     * an error
     * 
     * taken from BasicGUI included within the PER API
     */
    public boolean executeAction(Action a) {
        long last_update = 0;
        if(a == null)
            return false;
        
        System.out.println("About to execute action: " + a.getSummary());
        
        if (stop) {
            return false;
        }
        
        //start the action
        if (!a.doAction(rov))
            return false;
        
        //wait for the action to finish
        while (!stop && !a.isCompleted()) {
            //if the action takes pictures, display the latest image
            if( last_update != a.getImageUpdateTime()){
                last_update = a.getImageUpdateTime();
                //displayImage(a.getRecentImage());
            }
            try {
                Thread.sleep(10);
            } catch (java.lang.InterruptedException e) {
                System.out.println("executeAction interrupted");
            }
        }
        
        if (stop) {
        	System.out.println("Stopping the current action.");
            a.kill();
        }
        
        System.out.println("Action completed (" + ActionConstants.getErrorText(a.getReturnValue()) + ").");
        return (!stop && a.isSuccess());
    }
    
    
    /**
     * Tries to connect to the PER on the provided IP address
     * @param IP
     * @return
     */
    public boolean connectToPER(String IP) {
        boolean success;
        
        if (IP==null) { 
        	return false; 
        }
        //initialize communications
        success = rov.initComm(IP);
        rov.setCurrentIP(IP); //saves the current ip in case communication needs to be reestablished
        rov.saveIP(IP); //save the IP address so it can be loaded the next time the program is run
        
        if (success) {
        	success = rov.initRobot();
        	success = rov.refresh();
        	if (success) {
        		System.out.println("Connected to PER @ " + IP);
        		return true;
        	}
        	else {
        		System.out.println("Error: rover is not responding! Please check that the rover is on and ready, and then try again.");
        	}
        } 
        else {
        	System.out.println("Error: could not connect to the rover! Check that you have entered the IP address correctly. Also, make sure the rover is on and ready.");
        }
        return false;
    }
}
